from datetime import datetime

import pytz
from dash import callback_context, Dash
from flask import request
import os

CLIENT_DATE_FORMAT = "%Y-%m-%d %H:%M:%S"


def init_client_tz_javascript(dashapp: Dash):
    """
    Initializes a snippet of javascript which saves a cookie with the timezone of the client
    :param dashapp: The Dash app object
    """
    path = dashapp.config.get("assets_folder", "assets")

    if not os.path.exists(path):
        os.mkdir(path)
    with open(os.path.join(path, "timezone.js"), "w+") as f:
        f.write("document.cookie = 'tz=' + Intl.DateTimeFormat().resolvedOptions().timeZone;")


def get_client_tz() -> datetime.tzinfo:
    """
    Retrieves the timezone of the client, if it has been initialized. Otherwise returns UTC
    :return: The client timezone
    """
    return pytz.timezone(request.cookies.get("tz", "UTC"))


def to_client_tz(dt: datetime) -> datetime:
    """
    Transforms a datetime to the client timezone. Defaults to UTC if it has not been initialized.
    :param dt: A Datetime object
    :return: A datetime object localized in the client timezone
    """
    return dt.astimezone(get_client_tz())


def parse_client_date(s: str) -> datetime:
    """
    Parses a datetime and localizes it to the client timezone
    :param s: A string
    :return: A datetime object
    """
    dt = datetime.strptime(s, CLIENT_DATE_FORMAT)
    return get_client_tz().localize(dt)


def format_client_date(dt: datetime) -> str:
    """
    Formats a datetime object with a given format, in the clients timezone.
    :param dt: A datetime object
    :return: A string
    """
    tz = pytz.timezone(request.cookies.get("tz", "UTC"))
    return dt.astimezone(tz).strftime(CLIENT_DATE_FORMAT)


def was_source(name: str) -> bool:
    """
    Checks whether a given id triggered the callback
    :param name: Id of component
    :return: True if it was triggered by the given id
    """
    ctx = callback_context

    return ctx.triggered and  \
           ctx.triggered[0]['prop_id'].split('.')[0] == name


def get_source():
    """
    Returns the id of the object which triggered the callback
    :return: The id
    """
    ctx = callback_context

    return ctx.triggered[0]['prop_id'].split('.')[0]





from flask import session, abort, make_response

KEYCLOAK_CLIENT = "asset-control"


def has_read_access(subject):

    try:
        for x in session["introspect"]["resource_access"][KEYCLOAK_CLIENT]["roles"]:
            if x == "admin":
                return True
            name, level = x.split("/")
            if name in ["ALL", subject] and level == "READ":
                return True
    except KeyError:
        return False
    return False


def has_write_access(subject):
    try:
        for x in session["introspect"]["resource_access"][KEYCLOAK_CLIENT]["roles"]:
            if x == "admin":
                return True
            name, level = x.split("/")
            if name in ["ALL", subject] and level == "WRITE":
                return True
    except KeyError:
        return False
    return False


def check_role(role):
    return role in session["introspect"]["resource_access"][KEYCLOAK_CLIENT]["roles"]


def require_role(roles: list):
    def inner(func):
        def wrapper(*args, **kwargs):
            if session["introspect"]["resource_access"][KEYCLOAK_CLIENT]["roles"] in roles:
                func(*args, **kwargs)
            else:
                abort(make_response(f"You do not have the required access", 401))
        return wrapper
    return inner



from setuptools import setup
import pathlib

HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()


setup(
    name='ems-dash-utils',
    author="Jesper Halkjær Jensen",
    author_email="gedemagt@gmail.com",
    description="A utility package for dash apps",
    version='0.0.1',
    url='https://github.com/',
    packages=['dash_utils'],
    long_description=README,
    long_description_content_type="text/markdown",
    license="MIT",
    python_requires='>=3.6', install_requires=['dash', 'flask', 'pytz']
)
